--
-- MUTANTS : A generic genetic algorithm toolkit for Ada 95
--
-- Music Evaluation component
--
-- Author  Matthew Caryl
-- Created 21.2.97
--

with Mutants.Core.Evaluation;
with Mutants.Core.Representation;
with Mutants.Core.Representation.Bit_Vector;
with Mutants.Core.Representation.Bit;
with Ada.Text_IO;
use Mutants.Core.Evaluation;
use Mutants.Core.Representation;
use Mutants.Core.Representation.Bit_Vector;
use Mutants.Core.Representation.Bit;
use Ada.Text_IO;

package body Music_Evaluation is

	procedure PrintMusic(Eval : in Music_Evaluation; Rep : in out Vector_Representation) is
		representation_note : Natural := 1;
		direction : Integer := 1;
		bit : Bit_Representation_Access;
	begin
		for i in 1..Eval.Notes'Last loop
			case Eval.Notes(i).Rule is
				when 'N' =>
					null;
				when 'U' =>
					direction := 1;
				when 'D' =>
					direction := -1;
				when 'C' =>
					direction := -direction;
				when others =>
					Get(Rep, representation_note, bit);
					if Get(bit) then
						direction := -direction;
					end if;
					representation_note := representation_note + 1;
			end case;
			if direction = 1 then
				Put('U');
			else
				Put('D');
			end if;
		end loop;
	end PrintMusic;

	procedure PlayMusic(Eval : in Music_Evaluation; Rep : in out Vector_Representation; Length : out Natural) is
		positive_played_length : Integer := 0;
		negative_played_length : Integer := 0;
		current_position : Integer := 0;
		representation_note : Natural := 1;
		direction : Integer := 1;
		bit : Bit_Representation_Access;
	begin
		for i in 1..Eval.Notes'Last loop
			case Eval.Notes(i).Rule is
				when 'N' =>
					null;
				when 'U' =>
					direction := 1;
				when 'D' =>
					direction := -1;
				when 'C' =>
					direction := -direction;
				when others =>
					Get(Rep, representation_note, bit);
					if Get(bit) then
						direction := -direction;
					end if;
					representation_note := representation_note + 1;
			end case;
			current_position := current_position + Eval.Notes(i).Length * direction;
			if current_position > positive_played_length then
				positive_played_length := current_position;
			elsif current_position < negative_played_length then
				negative_played_length := current_position;
			end if;
		end loop;
		Length := positive_played_length - negative_played_length;
	end PlayMusic;

	function Evaluate(Evaluation : in Music_Evaluation; Representation : in Representation_Access) return Float is
		length : Natural;
	begin
		PlayMusic(Evaluation, Vector_Representation_Access(Representation).all, length);
		if length < Evaluation.BowLength then
			return Float(100 * length) / Float(Evaluation.BowLength);
		else
			return Float(100 * Evaluation.BowLength) / Float(length);
		end if;
	end Evaluate;
	
	procedure SetBowLength(Evaluation : in out Music_Evaluation; Length : in Natural) is
	begin
		Evaluation.BowLength := Length;
	end SetBowLength;

	procedure SetNote(Evaluation : in out Music_Evaluation; Which : Natural; Length : in Natural; Rule : Character) is
	begin
		Evaluation.Notes(Which).Length := Length;
		Evaluation.Notes(Which).Rule := Rule;
	end SetNote;

end Music_Evaluation;
