--
-- MUTANTS : a generic genetic algorithm toolkit for Ada 95
--
-- Core Toolkit - Population component
--
-- Author  Matthew Caryl
-- Created 14.12.96
--

with Mutants.Core.Individual;
with Ada.Finalization;
use Mutants.Core.Individual;

package Mutants.Core.Population is

	type Population is private;
	type Population_Access is access all Population;
	type PopulationIterator is private;
	
	procedure Add(Pop : in out Population; Individual : in Individual_Access);
	procedure Add(Pop : in out Population; Individuals : in Individual_Access_Array);
	procedure Add(Pop : in out Population; New_Population : Population);
	procedure Sub(Pop : in out Population; Individual : in Individual_Access);
	function Size(Pop : in Population) return Natural;
	procedure Copy(First, Second : in out Population);
	procedure Clear(Pop : in out Population);
	function Iterator(Pop : in Population_Access) return PopulationIterator;
	procedure Next(Iterator : in out PopulationIterator; More : out Boolean; Individual : out Individual_Access);

private

	-- average population size of 20 is just a wild guess
	type Population is new Ada.Finalization.Controlled with
		record
			Individuals : Individual_Access_Array_Access := new Individual_Access_Array(1..20);
			Size : Natural := 0;
		end record;

	procedure Adjust(Pop : in out Population);
	procedure Finalize(Pop : in out Population);

	type PopulationIterator is
		record
			Source : Population_Access;
			Place : Natural;
		end record;

end Mutants.Core.Population;
