--
-- MUTANTS : A generic genetic algorithm toolkit for Ada 95
--
-- Core toolkit - Random component
--
-- Author  Matthew Caryl
-- Created 10.12.96
--

with Ada.Numerics.Discrete_Random;

package body Mutants.Core.Random is

	procedure Reset(Random : in out Random) is
		package Random_Seed is new Ada.Numerics.Discrete_Random(Long_Integer);
		
		g : Random_Seed.Generator;
	begin
		Random_Seed.Reset(g);
		Random.Random_Seed := Random_Seed.Random(g);
	end Reset;
	
	procedure Reset(Random : in out Random; Seed : in Long_Integer) is
	begin
		Random.Random_Seed := Seed;
	end Reset;

	procedure Advance(Random : in Random_Access) is
	begin
		Random.Random_Seed := Random.Random_Seed * 1103515245 + 12345;
	end Advance;

	function Value(Random : in Random_Access) return Boolean is
	begin
		Advance(Random);
		return Random.Random_Seed > 0;
	end Value;

	function Value(Random : in Random_Access; Upper : in Integer) return Integer is
	begin
		Advance(Random);
		return Integer(Random.Random_Seed / 65535) mod Upper;
	end Value;

	function Value(Random : in Random_Access; Lower, Upper : in Integer) return Integer is
	begin
		Advance(Random);
		return Value(Random, Upper - Lower) + Lower;
	end Value;

	function Value(Random : in Random_Access; Upper, Precision : in Integer) return Float is
	begin
		Advance(Random);
		return Float(Value(Random, Upper * Precision)) / Float(Precision);
	end Value;

	function Value(Random : in Random_Access; Lower, Upper, Precision : in Integer) return Float is
	begin
		Advance(Random);
		return Float(Value(Random, (Upper - Lower) * Precision)) / Float(Precision) + Float(Lower);
	end Value;

end Mutants.Core.Random;
