--
-- MUTANTS : A generic genetic algorithm toolkit for Ada 95
--
-- Core toolkit - Order representation component
--
-- Author  Matthew Caryl
-- Created 4.2.97
--

generic
	type Element_Type is new Representation with private;
	type Element_Type_Access is access all Element_Type;
	with function Equal(one, two : Element_Type) return Boolean;

package Mutants.Core.Representation.Order is

	type Order_Representation(Size : Positive) is new Representation with private;
 	
 	function Equal(one, two : Order_Representation) return Boolean;

	function Get(Representation : in Order_Representation; Index : Positive) return Element_Type;
	procedure Set(Representation : in out Order_Representation; Index : Positive; Element : in Element_Type);
	function Size(Representation : in Order_Representation) return Positive;

	procedure Get(Representation : in out Order_Representation; Index : Positive; Element : out Element_Type_Access);

	type Order_Representation_Access is access all Order_Representation;

	function Get(Representation : in Order_Representation_Access; Index : Positive) return Element_Type;
	procedure Set(Representation : in Order_Representation_Access; Index : Positive; Element : in Element_Type);
	function Size(Representation : in Order_Representation_Access) return Positive;
	procedure Get(Representation : in Order_Representation_Access; Index : Positive; Element : out Element_Type_Access);

private

	type Element_Vector_Array is array (Standard.Integer range <>) of aliased Element_Type; 

	type Order_Representation(Size : Positive) is new Representation with
		record
			Element_Vector : Element_Vector_Array(1..Size);
		end record;

end Mutants.Core.Representation.Order;
