--
-- MUTANTS : a generic genetic algorithm toolkit for Ada 95
--
-- Core Toolkit - Roulette Wheel Selector component
--
-- Author  Matthew Caryl
-- Created 1.2.97
--

with Mutants.Core.Selector;
with Mutants.Core.Individual;
with Mutants.Core.Random;

use Mutants.Core.Selector;
use Mutants.Core.Individual;
use Mutants.Core.Random;

package body Mutants.Core.Selector.AntiRoulette is

        procedure Attach(Sel : in out AntiRoulette_Selector; Fit : in out Mutants.Core.Fitness.Fitness'Class) is
		fi : FitnessIterator'Class := Iterator(Fit);
		more : Boolean;
		i : Individual_Access;
		v : Float;
		max, min : Float;
	begin
		Sel.Fitness := Fit'Unchecked_Access;
		-- find maximum
		Next(fi, more, i, v);
		max := v;
		min := v;
		if not more then
			raise Empty_Population_Exception;
		end if;
		Next(fi, more, i, v);
		while more loop
			if v > max then
				max := v;
			elsif v < min then
				min := v;
			end if;
			Next(fi, more, i, v);
		end loop;
		-- find total
		Sel.Max_Min := max + min;
		fi := Iterator(Fit);
		Next(fi, more, i, v);
		Sel.Total := Sel.Max_Min - v;
		if not more then
			raise Empty_Population_Exception;
		end if;
		Next(fi, more, i, v);
		while more loop
			Sel.Total := Sel.Total + Sel.Max_Min - v;
			Next(fi, more, i, v);
		end loop;
	end Attach;

        procedure Get(Sel : in out AntiRoulette_Selector; Individual : out Individual_Access) is
		fi : FitnessIterator'Class := Iterator(Sel.Fitness.all);
		more : Boolean := true;
		v : Float;
		t : Float;
		c : Integer;
	begin
		if Sel.Total /= 0.0 then
			while true loop
				t := Value(Sel.Chance, Integer(Sel.Total), 10);
				Next(fi, more, individual, v);
				while more loop
					if t < Sel.Max_Min - v then
						return;
					end if;
					t := t - Sel.Max_Min + v;
					Next(fi, more, individual, v);
				end loop;
				fi := Iterator(Sel.Fitness.all);
			end loop;
		else
			c := 0;
			Next(fi, more , individual, v);
			while more loop
				c := c + 1;
				Next(fi, more, individual, v);
			end loop;
			c := Value(Sel.Chance, c);
			fi := Iterator(Sel.Fitness.all);
			while more loop
				c := c - 1;
				if c < 0 then
					return;
				end if;
				Next(fi, more, individual, v);
			end loop;
		end if;
	end Get;

        procedure Put(Sel : in out AntiRoulette_Selector; Individual : in Individual_Access) is
	begin
		null;
	end Put;

end Mutants.Core.Selector.AntiRoulette;
