--
-- MUTANTS : a generic genetic algorithm toolkit for Ada 95
--
-- Travelling Salesman Problem using MUTANTS
--
-- Author  Matthew Caryl
-- Created 5.2.96
--

with Mutants.Core.Random;
with Mutants.Core.Ending.Count;
with Mutants.Core.Ending.Time;
with Mutants.Core.Representation.Integer;
with Mutants.Extended.Representation.Integer_Order;
with TSP_Evaluation;
with Mutants.Core.Evaluation.One;
with Mutants.Core.Fitness.Unity;
with Mutants.Core.Selector.AntiRoulette;
with Mutants.Core.Selector.Cyclic;
with Mutants.Extended.Creator.Integer_Order_Initialise;
with Mutants.Extended.Creator.Integer_Order_Random_Reproduction;
with Mutants.Extended.Creator.Integer_Order_Mutation_Reproduction;
with Mutants.Extended.Creator.Integer_Order_Crossover_Reproduction;
with Mutants.Core.Creator.Bxor;
with Mutants.Core.Model.Generation;
with Mutants.Core.Population;
with Mutants.Core.Individual;
with Mutants.Core.Selector.Low;
with Ada.Numerics.Elementary_Functions;
with Ada.Text_IO.Float_IO;
with Ada.Text_IO.Integer_IO;
use Mutants.Core.Random;
use Mutants.Core.Ending.Count;
use Mutants.Core.Ending.Time;
use Mutants.Core.Representation;
use Mutants.Core.Representation.Integer;
use Mutants.Extended.Representation.Integer_Order;
use TSP_Evaluation;
use Mutants.Core.Evaluation;
use Mutants.Core.Evaluation.One;
use Mutants.Core.Fitness.Unity;
use Mutants.Core.Selector.AntiRoulette;
use Mutants.Core.Selector.Cyclic;
use Mutants.Extended.Creator.Integer_Order_Initialise;
use Mutants.Extended.Creator.Integer_Order_Random_Reproduction;
use Mutants.Extended.Creator.Integer_Order_Mutation_Reproduction;
use Mutants.Extended.Creator.Integer_Order_Crossover_Reproduction;
use Mutants.Core.Creator.Bxor;
use Mutants.Core.Model;
use Mutants.Core.Model.Generation;
use Mutants.Core.Population;
use Mutants.Core.Individual;
use Mutants.Core.Selector.Low;
use Ada.Text_IO;

procedure TSP is

	type TSP_Evaluation_Access is access all TSP_Evaluation.TSP_Evaluation;
	
	package Float_IO is new Ada.Text_IO.Float_IO(Float);
	package Integer_IO is new Ada.Text_IO.Integer_IO(Integer);

	use Float_IO;
	use Integer_IO;
	use Ada.Numerics.Elementary_Functions;
	function Get_Evaluation return TSP_Evaluation_Access is
		function Sqr(n : Float) return Float is
		begin
			return n * n;
		end Sqr;

		type Coord is
			record
				x, y : Float;
			end record;
		type Cities is array(Integer range <>) of Coord;
		type Cities_Access is access Cities;

		file : File_Type;
		size : Integer;
		c : Cities_Access;
		dummy : Integer;
		eval : TSP_Evaluation_Access;
	begin
		-- read in cities
		Open(file, In_File, "ulysses16.tsp");
		Skip_Line(file);
		Skip_Line(file);
		Skip_Line(file);
		Set_Col(file, 11);
		Get(file, size);
		eval := new TSP_Evaluation.TSP_Evaluation(size);
		Skip_Line(file);
		Skip_Line(file);
		Skip_Line(file);
		Skip_Line(file);
		c := new Cities(1..size);
		for i in 1..size loop
			Get(file, dummy);
			Get(file, c(i).x);
			Get(file, c(i).y);
		end loop;
		-- calculate distances
		for i in 1..size loop
			for j in 1..size loop
				Put(eval.all, Sqrt(Sqr(c(i).x - c(j).x) + Sqr(c(i).y - c(j).y)), i, j);
			end loop;
		end loop;
		Close(file);
		return eval;
	end Get_Evaluation;

	use Float_IO;
	use Integer_IO;
	function Get_Optimum return Individual_Access is
		file : File_Type;
		size : Integer;
		o : Order_Representation_Access;
		i : Integer_Representation;
		v : Integer;
		ind : Individual_Access;
	begin
		Open(file, In_File, "ulysses16.opt.tour");
		Skip_Line(file);
		Skip_Line(file);
		Skip_Line(file);
		Set_Col(file, 12);
		Get(file, size);
		Skip_Line(file);
		Skip_Line(file);
		o := new Order_Representation(size);
		for j in 1..size loop
			Get(file, v);
			Set(i, v);
			Set(o, j, i);
		end loop;
		Close(file);
		ind := new Mutants.Core.Individual.Individual(Representation_Access(o));
		return ind;
	end Get_Optimum;

	use Float_IO;
	procedure PrintIndividual(i : Individual_Access; e : Evaluation_Class) is
		o : Order_Representation_Access;
		v : Integer_Representation_Access;
	begin
		o := Order_Representation_Access(Mutants.Core.Individual.Representation(i));
		for j in 1..16 loop
			Get(o, j, v);
			Put(Get(v), Width=>0);
			Put(" ");
		end loop;
		New_Line;
		Put(Evaluate(e, Representation_Access(o)));
		New_Line;
	end PrintIndividual;

	procedure PrintBest(m : Model'Class; e : Evaluation_Class) is
		p : Population_Access;
		f : Unity_Fitness;
		s : Low_Selector;
		i : Individual_Access;
	begin
		Get_Population(m, p);
		EvaluatePopulation(p, e);
		Attach(f, p);
		Attach(s, f);
		Get(s, i);
		PrintIndividual(i, e);
	end PrintBest;

	size : Natural := 10;
	rand : aliased Random;
	order_initialise : aliased Integer_Order_Initialise(16);
	generational : Generation_Model(order_initialise'Unchecked_Access, size);
	one : One_Evaluation;
	evaluator : TSP_Evaluation_Access;
	unity : Unity_Fitness;
	cyclic : aliased Cyclic_Selector(16);
	randomise : Order_Random_Reproduction(rand'Unchecked_Access);
	selector : AntiRoulette_Selector(rand'Unchecked_Access);
	order_mutation : aliased Order_Mutation_Reproduction(rand'Unchecked_Access);
	order_crossover : aliased Order_Crossover_Reproduction(rand'Unchecked_Access);
	reproduction : Xor_Creator(order_mutation'Unchecked_Access, 1, order_crossover'Unchecked_Access, 3, rand'Unchecked_Access);
	count : Count_Ending;
	time : Time_Ending;
	population : Population_Access;
	optimum : Individual_Access;
begin
	-- randomise representation order
	Reset(count, 1);
	Get_Population(generational, population);
	Attach(unity, population);
	Attach(cyclic, unity);
	Run(generational, one, unity, cyclic, randomise, count);

	evaluator := Get_Evaluation;
	PrintBest(generational, evaluator.all);
	-- 100 samples
	for i in 1..100 loop
		Reset(count, 1);
		Run(generational, evaluator.all, unity, selector, reproduction, count);
		PrintBest(generational, evaluator.all);
	end loop;
	
	-- find optimum from file
	New_Line;
	New_Line;
	optimum := Get_Optimum;
	PrintIndividual(optimum, evaluator.all);
end TSP;
